"use strict";
var ESLintCommands;
(function (ESLintCommands) {
    ESLintCommands.GetErrors = "GetErrors";
    ESLintCommands.FixErrors = "FixErrors";
})(ESLintCommands || (ESLintCommands = {}));
var ESLintResponse = (function () {
    function ESLintResponse() {
    }
    return ESLintResponse;
}());
var ESLintPlugin = (function () {
    function ESLintPlugin(state) {
        this.additionalRulesDirectory = state.additionalRootDirectory;
        this.extraOptions = state.extraOptions;
        this.findESLint(state.eslintPackagePath);
        this.options = require(this.basicPath + "lib/options");
        this.CliEngine = require(this.basicPath + "lib/cli-engine");
        this.version = (this.linter == null) ? "Unknown" : this.linter.version;
    }
    ESLintPlugin.prototype.onMessage = function (p, writer) {
        var request = JSON.parse(p);
        var response = new ESLintResponse();
        response.request_seq = request.seq;
        response.command = request.command;
        response.version = this.version;
        try {
            if (this.linter != null && this.options != null && this.CliEngine != null) {
                if (request.command === ESLintCommands.GetErrors) {
                    response.body = this.getErrors(request.arguments);
                }
                else if (request.command === ESLintCommands.FixErrors) {
                    response.body = this.fixErrors(request.arguments);
                }
            }
        }
        catch (e) {
            response.error = e.toString() + "\n\n" + e.stack;
        }
        finally {
            writer.write(JSON.stringify(response));
        }
    };
    ESLintPlugin.prototype.getErrors = function (getErrorsArguments) {
        var args = this.createArguments(getErrorsArguments);
        var parsedOptions = this.options.parse(args);
        var cliEngine = new this.CliEngine(ESLintPlugin.translateOptions(parsedOptions));
        var report = cliEngine.executeOnText(getErrorsArguments.content, getErrorsArguments.fileName, true);
        return ESLintPlugin.formatResults(report, cliEngine, parsedOptions);
    };
    ESLintPlugin.formatResults = function (report, cliEngine, parsedOptions) {
        var output = cliEngine.getFormatter(parsedOptions.format)(report.results);
        // todo: too many warnings count
        return JSON.parse(output);
    };
    ESLintPlugin.prototype.fixErrors = function (fixErrorsArguments) {
        var args = this.createArguments(fixErrorsArguments);
        args += " --fix " + fixErrorsArguments.fileName;
        var parsedOptions = this.options.parse(args);
        var cliEngine = new this.CliEngine(ESLintPlugin.translateOptions(parsedOptions));
        var report = cliEngine.executeOnFiles(parsedOptions._);
        this.CliEngine.outputFixes(report);
        return ESLintPlugin.formatResults(report, cliEngine, parsedOptions);
    };
    ESLintPlugin.prototype.createArguments = function (getErrorsArguments) {
        var args = "";
        if (getErrorsArguments.configPath != null) {
            args += "-c \"" + getErrorsArguments.configPath + "\"";
        }
        if (this.extraOptions != null && this.extraOptions.length > 0) {
            args += " " + this.extraOptions;
        }
        if (this.additionalRulesDirectory != null && this.additionalRulesDirectory.length > 0) {
            args += " --rulesdir=\"" + this.additionalRulesDirectory + "\"";
        }
        return args;
    };
    ESLintPlugin.prototype.findESLint = function (eslintPackagePath) {
        if (eslintPackagePath.charAt(eslintPackagePath.length - 1) !== '/' &&
            eslintPackagePath.charAt(eslintPackagePath.length - 1) !== '\\') {
            eslintPackagePath = eslintPackagePath + '/';
        }
        eslintPackagePath = eslintPackagePath.replace("\\", "/");
        this.basicPath = eslintPackagePath;
        this.linter = require(eslintPackagePath + "lib/cli.js");
    };
    // taken from privtae part of eslint, we need it here
    /**
     * Translates the CLI options into the options expected by the CLIEngine.
     * @param {Object} cliOptions The CLI options to translate.
     * @returns {CLIEngineOptions} The options object for the CLIEngine.
     * @private
     */
    ESLintPlugin.translateOptions = function (cliOptions) {
        return {
            envs: cliOptions.env,
            extensions: cliOptions.ext,
            rules: cliOptions.rule,
            plugins: cliOptions.plugin,
            globals: cliOptions.global,
            ignore: cliOptions.ignore,
            ignorePath: cliOptions.ignorePath,
            ignorePattern: cliOptions.ignorePattern,
            configFile: cliOptions.config,
            rulePaths: cliOptions.rulesdir,
            useEslintrc: cliOptions.eslintrc,
            parser: cliOptions.parser,
            parserOptions: cliOptions.parserOptions,
            cache: cliOptions.cache,
            cacheFile: cliOptions.cacheFile,
            cacheLocation: cliOptions.cacheLocation,
            fix: cliOptions.fix,
            allowInlineConfig: cliOptions.inlineConfig
        };
    };
    return ESLintPlugin;
}());
exports.ESLintPlugin = ESLintPlugin;
